#version 430 core
out vec4 FragColor;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
    vec4 FragPosLightSpace;
	vec3 lightDir;
	} fs_in;

uniform sampler2D diffuseTexture;
uniform sampler2D shadowMap;

uniform mat4 projection;
uniform mat4 view;
uniform float multiplier;
uniform vec3 lightPos;
uniform vec3 viewPos;

uniform vec4 tint;

uniform bool secondRun;

const float levels = 5.0;

void cel(inout float value)
{
	float levels = 6.0;

	float val = floor(value * levels);
	val = val / levels;

	value = val;

}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

float test;

float ShadowCalculation(vec4 fragPosLightSpace)
{	
	test = 1.0;

    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // calculate bias (based on depth map resolution and slope)
    vec3 normal = normalize(fs_in.Normal);
    //vec3 lightDir = normalize(lightPos - fs_in.FragPos);
	//vec3 lightDir = normalize(fs_in.FragPos - lightPos);
    float bias = max(0.0001* (1.0 - dot(normal, fs_in.lightDir)), 0.0001);
    // check whether current frag pos is in shadow
    // float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;
    // PCF
    float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;
			
        }    
    }
    shadow /= 9.0;
    
    // keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0)
        shadow = 0.0;
        
		test = projCoords.z;
    return shadow;
}

void main()
{           
    vec3 color = texture(diffuseTexture, fs_in.TexCoords).rgb;

	float r=color.r;
	float g=color.g;
	float b=color.b;

	//rgb to hsv
	 float K = 0.0;
     float tmp;

	  if (g < b)
     {
         tmp = g;
         g=b;
         b=tmp;

         K = -1.0;
     }

     if (r < g)
     {
         tmp = r;
         r=g;
         g=tmp;

         K = -2.0 / 6.0 - K;
     }

     float chroma = r - min(g, b);

     color.r = abs(K + (g - b) / (6.0 * chroma + 1e-20));
     color.g = chroma / (r + 1e-20);
     color.b = r;

	 cel(color.b);

	 color = hsv2rgb(color);

    vec3 normal = normalize(fs_in.Normal);

	// color of light 1 = white, 0 = black
    vec3 lightColor = vec3(0.9);

    // ambient - general brightness
    vec3 ambient = 0.5 * color * multiplier;

    // diffuse
    vec3 lightDir = normalize(lightPos - fs_in.FragPos);
    float diff = max(dot(lightDir, normal), 0.0);
	//float diff_level = floor (diff * levels);
	//diff = diff_level/levels;
	//cel(diff);
    vec3 diffuse = diff* lightColor;

    // specular
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = 0.0;
    vec3 halfwayDir = normalize(lightDir + viewDir);  
    spec = pow(max(dot(normal, halfwayDir), 0.0), 64.0);
	//float level = floor (spec*levels);
	//spec = level/levels;
	//cel(spec);
    vec3 specular = spec * lightColor;    

    // calculate shadow
    float shadow = ShadowCalculation(fs_in.FragPosLightSpace);

	
    vec3 lighting = (ambient + (1.0 - shadow) * (diffuse + specular)) * color * (secondRun? 0 : 1);  
	

		FragColor = vec4(lighting, 1.0) + tint * tint.w;
	
}